﻿using WingRpcSDK.Common.Loader;
using WingRpcSDK.JsonRpc.Driver;
using JsonRpcLite.InProcess;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;

namespace WingCloudServer
{
    internal abstract class BaseServiceLoader
    {
        private static AssemblyLoader _assemblyLoader;
        internal static object RestartLocker = new object();

        /// <summary>
        /// RpcInProcessClient
        /// </summary>
        internal static InProcessRpcClient RpcInProcessClient { get; }

        /// <summary>
        /// InProcessServer
        /// </summary>
        /// <value></value>
        internal static RpcServer InProcessServer { get; }

        /// <summary>
        /// InProcessEngine
        /// </summary>
        /// <value></value>
        internal static JsonRpcInProcessEngine InProcessEngine { get; }

        static BaseServiceLoader()
        {
            InProcessEngine = new JsonRpcInProcessEngine();
            InProcessServer = new RpcServer(InProcessEngine);
            RpcInProcessClient = InProcessRpcClient.CreateClient(InProcessEngine);
            _assemblyLoader = new AssemblyLoader();
        }

        /// <summary>
        /// Create service
        /// </summary>
        /// <typeparam name="T">Service convert type</typeparam>
        /// <param name="serviceType">Create source type</param>
        /// <returns>return need type created instance</returns>
        private T CreateLoader<T>(Type serviceType)
        {
            var service = (T)Activator.CreateInstance(serviceType);
            return service;
        }

        /// <summary>
        /// Register service
        /// </summary>
        /// <typeparam name="T">Service type</typeparam>
        /// <param name="service">Service instance</param>
        /// <param name="rpcService">Run rpc service instance</param>
        /// <param name="interfaceType">The service implement from interface type</param>
        internal void CallRegister(RpcServer rpcServer, Type serviceType, object service)
        {
            rpcServer.RegisterService(serviceType, service);
        }

        /// <summary>
        /// Get the service base type,used to create and register service  
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="filePath"></param>
        /// <returns></returns>
        internal IEnumerable<Type> GetLoaders<T>(string filePath)
        {
            var assembly = _assemblyLoader.LoadAssemblyDependencies(filePath);
            var allTypes = assembly.GetTypes();
            return allTypes.Where(t => t.GetInterface(typeof(T).Name) != null);
        }

        /// <summary>
        /// Load service dll
        /// </summary>
        internal virtual void LoadModules(string path)
        {
            if (Directory.Exists(path))
            {
                var dlls = Directory.GetFiles(path, "*.dll");
                foreach (var file in dlls)
                {
                    if (file.IndexOf("Module") > -1)
                    {
                        var serviceLoaders = GetLoaders<IServiceLoader>(file);
                        foreach (var serviceLoader in serviceLoaders)
                        {
                            var loader = CreateLoader<IServiceLoader>(serviceLoader);
                            var serviceType = loader.GetServiceType();
                            var service = loader.CreateService();
                            RegisterService(serviceType, service);
                        }
                    }
                }
                var dirs = Directory.GetDirectories(path);
                foreach (var dir in dirs)
                {
                    LoadModules(dir);
                }
            }
        }

        /// <summary>
        /// Start service
        /// </summary>
        internal abstract void StartService();

        /// <summary>
        /// Stop service
        /// </summary>
        internal abstract void StopService();

        /// <summary>
        /// Register servcie
        /// </summary>
        /// <param name="serviceType">service type</param>
        /// <param name="service">instance</param>
        internal abstract void RegisterService(Type serviceType, object service);
    }
}
